/*
 * Contributor(s): Natasha Noy noy@smi.stanford.edu
 */
package edu.stanford.smi.fmatoowl;

import java.io.*;
import java.util.*;

import com.hp.hpl.jena.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.owl.*;
import edu.stanford.smi.protegex.owl.jena.*;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.impl.*;
import edu.stanford.smi.protegex.owl.model.triplestore.*;
import edu.stanford.smi.protegex.owl.model.util.*;

public class FMAtoOWL {
	private static Project _sourceProject = null;
	private static KnowledgeBase _sourceKB = null;

	private static boolean _processingDL = true;
//	private static HashMap _owlClassNames = new HashMap (); //<owl label, owl class>
//	private static HashMap _owlPropertyNames = new HashMap (); //<owl property label, owl property>
	
	private static JenaOWLModel _currentModel = null;
	private static JenaOWLModel _importedModel = null;
	
	private static String _dlNameSpace = null;
	private static String _fullNameSpace = null;

	// arg[0]: path to the project project file
	// arg[1] (optional): dl namespace
	// arg[2] (optional): full namespace
	public static void main(String[] args) {
		Collection errors = new ArrayList();
		Log.getLogger().info("Start conversion at " + new Date ());
		
		String sourceFileName = args.length > 0 ? args[0] : Constants.SOURCE_FILE_NAME;
		_dlNameSpace = args.length > 1 ? args[1] : Constants.FMA_IN_OWL_DL_NAMESPACE;
		_fullNameSpace = args.length > 2 ? args[2] : Constants.FMA_IN_OWL_FULL_NAMESPACE;
		
		_sourceProject = new Project(sourceFileName, errors);
		if (errors.size() != 0) {
			displayErrors(errors);
			return;
		}
		
		_sourceKB = _sourceProject.getKnowledgeBase();
		
		
		
			createDLComponent ();
			_processingDL = false;
			createFullComponent ();

		Log.getLogger().info("DONE: " + new Date());
	}
	
	private static void createDLComponent () {
		Collection errors = new ArrayList();
		
		_currentModel = ProtegeOWL.createJenaOWLModel();
		_currentModel.getNamespaceManager().setDefaultNamespace(_dlNameSpace);
		
		Properties properties  = new Properties (_currentModel, _sourceKB);
		properties.createProperties ();
		
		OWLDLComponent dlDlComponent = new OWLDLComponent (_currentModel, _sourceKB, false);
		dlDlComponent.createDLComponent (Constants.ROOT_CLASS_NAMES);
		
		String fileName = Constants.OWL_DL_FILE_NAME;
		_currentModel.save(new File(fileName).toURI(), FileUtils.langXMLAbbrev, errors);
		Log.getLogger().info("DL file saved with " + errors.size() + " errors.");
	}
	
	private static void createFullComponent () {
		_currentModel = ProtegeOWL.createJenaOWLModel();
		_currentModel.getNamespaceManager().setDefaultNamespace(_fullNameSpace);
		_currentModel.getNamespaceManager().setPrefix(_dlNameSpace, Constants.DL_PREFIX);
		ImportHelper importHelper = new ImportHelper (_currentModel);
		try {
			importHelper.addImport(new File(Constants.OWL_DL_FILE_NAME).toURI());
			importHelper.importOntologies();
		} catch (Exception e) {
			Log.getLogger().warning(e.toString());
		}
		
		setImportedModel ();
		
//		OWLDLComponent fullDlComponent = new OWLDLComponent (_currentModel, _sourceKB, true);
//		fullDlComponent.createDLComponent (Constants.OWL_FULL_ROOT_CLASS_NAMES);
		
		OWLFullComponent fullComponent = new OWLFullComponent (_currentModel, _sourceKB);
		fullComponent.createFullComponent(getAllRoots ());		
		
		Collection errors = new ArrayList();
		String fullFileName = Constants.OWL_FULL_FILE_NAME;
		_currentModel.save(new File(fullFileName).toURI(), FileUtils.langXMLAbbrev, errors);
		Log.getLogger().info("File saved with " + errors.size() + " errors.");
		
	}
	
	
	private static String[] getAllRoots() {
		int numberOfRoots = Constants.OWL_DL_ROOT_CLASS_NAMES.length + Constants.OWL_FULL_ROOT_CLASS_NAMES.length;
		String [] result = new String [numberOfRoots];
		int i = 0;
		for (; i < Constants.OWL_FULL_ROOT_CLASS_NAMES.length; i++) {
			result [i] = 	Constants.OWL_FULL_ROOT_CLASS_NAMES [i];		
		}
		for (; i < numberOfRoots; i++) {
			result [i] = Constants.OWL_DL_ROOT_CLASS_NAMES [i - Constants.OWL_FULL_ROOT_CLASS_NAMES.length];
		}
		return result;
	}
	
	private static void setImportedModel () {
		OWLOntology activeOntology = OWLUtil.getActiveOntology(_currentModel);
		Collection ontologies = _currentModel.getOWLOntologies();
		
		Iterator i = ontologies.iterator();
		while (i.hasNext()) {
			OWLOntology next = (OWLOntology)i.next();
			if (next != activeOntology) {
				_importedModel = (JenaOWLModel)next.getOWLModel();
			}
		}
	}
	
/*	private static void changeActiveOntology() {
		OWLOntology activeOntology = OWLUtil.getActiveOntology(_fullOwlModel);
		Collection ontologies = _fullOwlModel.getOWLOntologies();
		
		Iterator i = ontologies.iterator();
		while (i.hasNext()) {
			OWLOntology next = (OWLOntology)i.next();
			if (next != activeOntology) {
				TripleStoreModel tsm = _fullOwlModel.getTripleStoreModel();
				TripleStore tripleStore = tsm.getHomeTripleStore(next);
				tsm.setActiveTripleStore(tripleStore);
				tsm.updateEditableResourceState();
			}
		}
		
	}
*/	
	private static void displayErrors(Collection errors) {
		Iterator i = errors.iterator();
		while (i.hasNext()) {
			System.out.println("Error: " + i.next());
		}
	}
	
//	public static void addOWLClassName (Cls cls, OWLNamedClass owlCls) {
//		_owlClassNames.put(owlCls.getName(), owlCls);
//		_frameToOwlMap.put(cls.getName(), owlCls);
//	}
	
//	public static Collection getOwlClasses () {
//		return _owlClassNames.values();
//	}
//	
	public static OWLNamedClass getOwlClass (Cls cls) {
		if (cls.isSystem()) return null;
		String name = getValidOWLName (cls.getName());
		OWLNamedClass result = _currentModel.getOWLNamedClass(name);
		if (result == null && _importedModel != null)
			return _importedModel.getOWLNamedClass (Constants.DL_PREFIX + ":" + name);
		else
			return result;
	}
	
//	public static void addOWLPropertyName (Slot slot, OWLProperty property) {
//		_owlPropertyNames.put(property.getName(), property);
//		_frameToOwlMap.put(slot.getName(), property);
//	}
//	
	public static OWLProperty getOWLProperty (Slot slot) {
		if (slot.isSystem()) return null;
		String name = getValidOWLName (slot.getName());
		OWLProperty result = _currentModel.getOWLProperty(name);
		if (result == null && _importedModel != null)
			return _importedModel.getOWLProperty (Constants.DL_PREFIX + ":" + name);
		else
			return result;
	}
	
	public static OWLProperty getOWLProperty (String name) {
		OWLProperty result = _currentModel.getOWLProperty(name);
		if (result == null && _importedModel != null)
			return _importedModel.getOWLProperty (Constants.DL_PREFIX + ":" + name);
		else
			return result;
	}
	
	public static String getValidOWLName(String name) {
		char first = name.charAt (0);
		if (first >= '0' && first <= '9')
			name = "_" + name;
		name = name.replaceAll("\\W", "_");
		return name;
	}
	
	public static Collection getOWLClasses(Collection clses) {
		if (clses == null) return null;
		Collection result = new ArrayList ();
		Iterator i = clses.iterator();
		while (i.hasNext()) {
			Cls next = (Cls)i.next();
			OWLNamedClass nextOwlClass = getOwlClass (next);
			if (nextOwlClass != null)
				result.add(nextOwlClass);
		}
		return result;
	}
	
	
	
}