/*
 * Contributor(s): Natasha Noy noy@smi.stanford.edu
 */
package edu.stanford.smi.fmatoowl;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.owl.jena.*;
import edu.stanford.smi.protegex.owl.model.*;

public class OWLFullComponent {
	private KnowledgeBase _sourceKb = null;
	private JenaOWLModel _fullOwlModel = null;
	
	private HashSet _processedSlots = null;
	private SystemFrames _systemFrames = null;
	
	public OWLFullComponent (JenaOWLModel fullOWLModel, KnowledgeBase sourceKb) {
		_sourceKb = sourceKb;
		_systemFrames = new SystemFrames (_sourceKb);
		_fullOwlModel = fullOWLModel;		
	}
	
	public  void createFullComponent (String [] roots) {
		Log.getLogger().info("" + new Date () + " **** FULL COMPONENT ******");

		Vector classesInQueue = new Vector();

		HashSet classesProcessed = new HashSet (_sourceKb.getClsCount());
		
		for (int r = 0; r < roots.length; r++)
			classesInQueue.add(_sourceKb.getCls(roots [r]));
		
		initializeProcessedSlots ();
		
		int counter = 0;
		
		while (!classesInQueue.isEmpty()) {
        		Cls next = (Cls)classesInQueue.firstElement();
        		classesInQueue.remove(0);
        		
        		if (classesProcessed.contains(next)) continue;
        		classesProcessed.add(next);
        	
        		if (counter % 100 == 0)
        			Log.getLogger().info("Full component: " + counter + ":" + next);

        		OWLNamedClass nextOwlClass = FMAtoOWL.getOwlClass (next);
         	
        		copyOWLFullInformation (next, nextOwlClass);
 
        		Collection nextSubs = next.getDirectSubclasses();
        		if (nextSubs != null)
        			classesInQueue.addAll(0,  nextSubs);
        		
         		counter++;
        }
		
	
	}
	
	// need to add subclasses to the classes that were not there on the first pass
	private void addSubclasses(Cls cls, OWLNamedClass owlClass) {
		Collection subs = cls.getDirectSubclasses(); 
		Iterator i = subs.iterator();
		while (i.hasNext()) {
			Cls next = (Cls)i.next();
			OWLNamedClass owlSub = FMAtoOWL.getOwlClass(next);
			if (owlSub != null) {
				try {
					owlSub.addSuperclass(owlClass);
				} catch (Exception e) {
					Log.getLogger().warning("Triple store switching porblems. owlSub = " + owlSub + ", owlClass " + owlClass);
				}
			}
		}
		
	}

	private void addSystemSuperClasses(Cls cls, OWLNamedClass owlClass) {
		if (cls.getDirectSuperclasses().contains(_systemFrames.getStandardClsMetaCls())) {
			owlClass.addSuperclass(_fullOwlModel.getOWLNamedClassClass());
		}
		if (!cls.getDirectSuperclasses().contains(_sourceKb.getRootCls()) &&
				owlClass.getSuperclasses(false).contains(_fullOwlModel.getOWLThingClass())) {
			owlClass.removeSuperclass(_fullOwlModel.getOWLThingClass());
		}
		
	}

	private void copyOWLFullInformation (Cls source, OWLNamedClass target) {
		addSystemSuperClasses (source, target);
		copyMetaclass (source, target);
		Util.copyOwnSlotValues (_fullOwlModel, (Instance)source, (RDFResource)target, _processedSlots);
		copyTemplateSlotValues (source, target);
		addSubclasses (source, target);
	}
	
	private void copyTemplateSlotValues (Cls source, OWLNamedClass target) {
		Collection directSlots = source.getTemplateSlots();
		if (directSlots == null || directSlots.isEmpty()) return;
		Iterator s = directSlots.iterator();
		while (s.hasNext()) {
			Slot nextSlot = (Slot)s.next();
			Collection overriddenFacets = source.getDirectlyOverriddenTemplateFacets(nextSlot);
			if (overriddenFacets == null || overriddenFacets.isEmpty()) continue;
			
			Iterator f = overriddenFacets.iterator();
			while (f.hasNext()) {
				Facet nextFacet = (Facet)f.next();
				if (nextFacet.getFrameID().equals(Model.FacetID.VALUES) &&
						(nextSlot.getValueType() == ValueType.CLS ||
				          nextSlot.getValueType() == ValueType.INSTANCE)) { // if the value was not a class, it was taken care of at the DL stage
					OWLProperty nextProperty = FMAtoOWL.getOWLProperty(nextSlot);
					if (nextProperty == null) continue;
					Collection values = source.getDirectTemplateFacetValues(nextSlot, nextFacet);
					if (values == null || values.isEmpty()) continue;
					Iterator v = values.iterator();
					while (v.hasNext()) {
						Object nextValue = v.next();
						if (nextValue instanceof Cls)
							nextValue = FMAtoOWL.getOwlClass((Cls)nextValue);
						target.addSuperclass(_fullOwlModel.createOWLHasValue(nextProperty, nextValue));
					}
					
				}
			}
		}
	}
	

	private void copyMetaclass(Cls source, OWLNamedClass target) {
		Cls type = source.getDirectType();
		if (type == _sourceKb.getRootClsMetaCls()) return;
		OWLNamedClass owlType = FMAtoOWL.getOwlClass(type);
		if (owlType != null) {
			target.setRDFType(owlType);		
		}
	}

//	private OWLNamedClass createOWLClass (String name, OWLNamedClass superCls, OWLNamedClass metaCls) {
//		OWLNamedClass result = _fullOwlModel.createOWLNamedSubclass(name, superCls);
//		
//		result.removeSuperclass(_owlThing);
//		return result;
//	}
	
	private void initializeProcessedSlots () {
		_processedSlots = new HashSet (_sourceKb.getSlotCount());
		int i;
		for (i = 0; i < Constants.ANNOTATION_DATATYPE_PROPERTIES.length; i++)
			_processedSlots.add(_sourceKb.getSlot(Constants.ANNOTATION_DATATYPE_PROPERTIES[i]));
		for (i = 0; i < Constants.ANNOTATION_OBJECT_PROPERTIES.length; i++)
			_processedSlots.add(_sourceKb.getSlot(Constants.ANNOTATION_OBJECT_PROPERTIES[i]));
	}
}
