/*
 * Contributor(s): Natasha Noy noy@smi.stanford.edu
 */
package edu.stanford.smi.fmatoowl;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.owl.jena.*;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.impl.*;

public class Properties {
	private JenaOWLModel _owlModel = null;
	private KnowledgeBase _sourceKB = null;
	
	public Properties (JenaOWLModel dlOWLModel, KnowledgeBase sourceKb) {
		_owlModel = dlOWLModel;
		_sourceKB = sourceKb;
	}
	
	public void createProperties  () {
		Collection slots = new HashSet (_sourceKB.getSlots());
		createAnnotationProperties (slots, Constants.ANNOTATION_DATATYPE_PROPERTIES, false);
		createAnnotationProperties (slots, Constants.ANNOTATION_OBJECT_PROPERTIES, true);
		
		Iterator s = slots.iterator();
		while (s.hasNext()) {
			Slot next = (Slot)s.next();
			if (!next.isSystem()) {
				createProperty (next);
			}
		}
		processTopLevelSlotInformation ();	
		
	}
	
	private void createAnnotationProperties(Collection slots, String [] names, boolean objectProperties) {
		for (int i = 0; i < names.length; i++) {
			OWLProperty property = objectProperties ? (OWLProperty) _owlModel.createAnnotationOWLObjectProperty(FMAtoOWL.getValidOWLName(names [i])) : 
				(OWLProperty) _owlModel.createAnnotationOWLDatatypeProperty(FMAtoOWL.getValidOWLName(names [i]));
			Slot slot = _sourceKB.getSlot (names [i]);
//			FMAtoOWL.addOWLPropertyName (slot, property);
			slots.remove(slot);
		}
		
	}

	private void createProperty (Slot slot) {
		OWLProperty property = null;
		ValueType valueType = slot.getValueType();
		String propertyName = FMAtoOWL.getValidOWLName (slot.getName());
		if (valueType.equals(ValueType.BOOLEAN)) {
			property = _owlModel.createOWLDatatypeProperty(propertyName, _owlModel.getXSDboolean());
		} else if (valueType.equals(ValueType.FLOAT)) {
			property = _owlModel.createOWLDatatypeProperty(propertyName, _owlModel.getXSDfloat());
		} else if (valueType.equals(ValueType.INTEGER)) {
			property = _owlModel.createOWLDatatypeProperty(propertyName, _owlModel.getXSDinteger());
		} else if (valueType.equals(ValueType.STRING)) {
			property = _owlModel.createOWLDatatypeProperty(propertyName, _owlModel.getXSDstring());
		} else if (valueType.equals(ValueType.SYMBOL)) {
			Collection symbolValues = slot.getAllowedValues();
			RDFSLiteral [] literals = new RDFSLiteral [symbolValues.size()];
			Iterator v = symbolValues.iterator();
			for (int i = 0 ; v.hasNext(); i++) {
				String next = (String)v.next();
				literals[i] = new DefaultRDFSLiteral (_owlModel, next);
			}
			property = _owlModel.createOWLDatatypeProperty(propertyName, literals);
//			property.setRange(_dlOWLModel.getXSDstring());
			
		} else if (valueType.equals(ValueType.INSTANCE) || valueType.equals(ValueType.CLS)) {
			property = _owlModel.createOWLObjectProperty(propertyName);
		}
//		FMAtoOWL.addOWLPropertyName (slot, property);
	}
	
	// missing top level information: cardinality, if something other than single
	private void processTopLevelSlotInformation () {
		Collection slots = _sourceKB.getSlots();
		
		for (Iterator s = slots.iterator(); s.hasNext(); ) {
			Slot next = (Slot)s.next();
			if (next.isSystem()) continue;

			OWLProperty nextProperty = FMAtoOWL.getOWLProperty(next);
			
			if (next.getMaximumCardinality() == 1)
				nextProperty.setFunctional(true);
			
			// copy slot documentation
			Collection nextSlotDocumentation = next.getDocumentation();
			if (nextSlotDocumentation != null)
				nextProperty.setComment((String)CollectionUtilities.getFirstItem (nextSlotDocumentation));
			
			for (int i = 0; i < Constants.ANNOTATION_DATATYPE_PROPERTIES.length; i++) {
				Slot slot = _sourceKB.getSlot(Constants.ANNOTATION_DATATYPE_PROPERTIES[i]);
				OWLProperty property = FMAtoOWL.getOWLProperty(slot);
				Collection values = next.getOwnSlotValues(slot);
				if (values != null) {
					Iterator v = values.iterator();
					while (v.hasNext()) {
						String nextValue = (String)v.next();
						nextProperty.addPropertyValue(property, new DefaultRDFSLiteral (_owlModel, nextValue));
					}
				}
			}
			
			// copy invereses			
			Slot nextInverse = next.getInverseSlot();
			if (nextInverse != null) {
				
				OWLProperty nextInverseProperty = FMAtoOWL.getOWLProperty(nextInverse);
				nextProperty.setInverseProperty(nextInverseProperty);
			}
		}
	}
}
