/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is FMAtoOWL
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 */
/*
 * Contributor(s): Natasha Noy noy@smi.stanford.edu
 */
package edu.stanford.smi.fmatoowl;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.owl.jena.*;
import edu.stanford.smi.protegex.owl.model.*;

public class Util {

	public static void copyOwnSlotValues(JenaOWLModel owlModel, Instance source, RDFResource target) {
		copyOwnSlotValues (owlModel, source, target, new HashSet ());
	}

	public static void copyOwnSlotValues(JenaOWLModel owlModel, Instance source, RDFResource target, Collection slotsToIgnore) {
		Collection slots = source.getOwnSlots();
		Iterator i = slots.iterator();
		while (i.hasNext()) {
			Slot next = (Slot)i.next();
			
			if (next.isSystem() || slotsToIgnore.contains(next)) continue;
			
			Collection values = source.getDirectOwnSlotValues(next);
			if (values == null || values.isEmpty()) continue;
			
			RDFProperty nextProperty = FMAtoOWL.getRDFProperty(next);
			
			Iterator v = values.iterator();
			while (v.hasNext()) {
				Object nextValue = v.next();
				Object owlValue =  copyOwnSlotValue (owlModel, nextValue);
				Collection currentValues = target.getPropertyValues(nextProperty);
				if (currentValues != null && currentValues.contains (owlValue)) continue;
	//			Log.getLogger().info("target = " + target + ", nextproperty = " + nextProperty + ", nextValue = " + nextValue);
				if (owlValue != null)
					target.addPropertyValue(nextProperty, owlValue);
				else
					Log.getLogger().warning("Class has no superclass: " + nextValue);
					// ** throws a Class cast exception inside  edu.stanford.smi.protege.model.Project.getInheritedBrowserSlotPattern
					// ** trying to update UI
				
	//			Log.getLogger().info("----");
			}
		}
	}

	private static Object copyOwnSlotValue (JenaOWLModel owlModel, Object value) {
		Object owlValue = value;
		if (value instanceof Cls) {
			owlValue = FMAtoOWL.getOwlClass((Cls)value);
		} else if (value instanceof Instance) {
			owlValue = copyInstance (owlModel, (Instance)value);
		}
		return owlValue;
	}

	public static Instance copyInstance (JenaOWLModel owlModel, Instance instance) {
		Cls type = instance.getDirectType();
		OWLNamedClass owltype = FMAtoOWL.getOwlClass(type);
//Log.getLogger().info("type = " + type + "owltypa = " + owltype);
		
		String newName = FMAtoOWL.getValidOWLName(instance.getName());
		RDFIndividual existing = owlModel.getRDFIndividual(newName);
		while (existing != null) {
			newName += "_";
			existing = owlModel.getRDFIndividual(newName);
		}
	
		//******** creates the individual in the wrong model
		//OWLIndividual result = owltype.createOWLIndividual(FMAtoOWL.getValidOWLName(newName));
		Instance result = owlModel.createInstance(newName, owltype);
		
		copyOwnSlotValues (owlModel, instance, (OWLIndividual)result);
		
		return result;
	}

	public static final int DOMAIN = 1;
	public static final int RANGE = 2;
	public static void copyDomainAndRangeValues (int domainOrRange, Slot slot, KnowledgeBase kb, RDFProperty property, JenaOWLModel owlModel) {
			Collection nextDomainsOrRanges = null;
			if (domainOrRange == DOMAIN) {
				//*** the following should work, but there is a Protege bug, and for many slots it returns an incomplete list
				// (e.g. regional part doesn't have Anatomical structure in its domain)
				//nextDomainsOrRanges = next.getDirectDomain();
				nextDomainsOrRanges = getDomains (slot, kb);
			} else if (slot.getValueType() == ValueType.INSTANCE) {
				nextDomainsOrRanges = slot.getAllowedClses();
			} else {
				// *** This works in the case of the FMA, but not in general
				nextDomainsOrRanges = slot.getAllowedParents();
			}
			
			Collection owlDomainsOrRanges = FMAtoOWL.getOWLClasses(nextDomainsOrRanges);
			if (owlDomainsOrRanges != null && !owlDomainsOrRanges.isEmpty()) {
				if (owlDomainsOrRanges.size() == 1) {
					OWLNamedClass owlDomainOrRange = (OWLNamedClass)CollectionUtilities.getSoleItem (owlDomainsOrRanges);
					if (domainOrRange == DOMAIN)
						property.setDomain(owlDomainOrRange);
					else
						property.setRange(owlDomainOrRange);
				} else {
					if (domainOrRange == DOMAIN)
						property.setDomain(owlModel.createOWLUnionClass(owlDomainsOrRanges));
					else
						property.setUnionRangeClasses(owlDomainsOrRanges);
				}
			}
	}

	//**** This is a workaround; slot.getDirectDomain sholud work instead
	//but there is a Protege bug, and for many slots it returns an incomplete list
	// (e.g. regional part doesn't have Anatomical structure in its domain)
	private static Collection getDomains (Slot slot, KnowledgeBase kb) {
		Collection result = new ArrayList ();
		Collection references = slot.getReferences();
		
		
		Iterator r = references.iterator();
		while (r.hasNext()) {
			Reference next = (Reference)r.next();
			if (!next.getSlot().equals(kb.getSlot(Model.Slot.DIRECT_TEMPLATE_SLOTS))) continue;
			result.add(next.getFrame());
		}
		
		return result;
	}


}
