/*
 * Contributor(s): Natasha Noy noy@smi.stanford.edu
 */
package edu.stanford.smi.fmatoowl;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Facet;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.model.Model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.owl.jena.*;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.impl.*;

public class Util {

	public static void copyOwnSlotValues(JenaOWLModel owlModel, Instance source, RDFResource target) {
		copyOwnSlotValues (owlModel, source, target, new HashSet ());
	}

	public static void copyOwnSlotValues(JenaOWLModel owlModel, Instance source, RDFResource target, Collection slotsToIgnore) {
		Collection slots = source.getOwnSlots();
		Iterator i = slots.iterator();
		while (i.hasNext()) {
			Slot next = (Slot)i.next();
			
			if (next.isSystem() || slotsToIgnore.contains(next)) continue;
			
			Collection values = source.getDirectOwnSlotValues(next);
			if (values == null || values.isEmpty()) continue;
			
			OWLProperty nextProperty = FMAtoOWL.getOWLProperty(next);
			
			Iterator v = values.iterator();
			while (v.hasNext()) {
				Object nextValue = v.next();
				Object owlValue =  copyOwnSlotValue (owlModel, nextValue);
				Collection currentValues = target.getPropertyValues(nextProperty);
				if (currentValues != null && currentValues.contains (owlValue)) continue;
	//			Log.getLogger().info("target = " + target + ", nextproperty = " + nextProperty + ", nextValue = " + nextValue);
				if (owlValue != null)
					target.addPropertyValue(nextProperty, owlValue);
				else
					Log.getLogger().warning("Class has no superclass: " + nextValue);
					// ** throws a Class cast exception inside  edu.stanford.smi.protege.model.Project.getInheritedBrowserSlotPattern
					// ** trying to update UI
				
	//			Log.getLogger().info("----");
			}
		}
	}

	private static Object copyOwnSlotValue (JenaOWLModel owlModel, Object value) {
		Object owlValue = value;
		if (value instanceof Cls) {
			owlValue = FMAtoOWL.getOwlClass((Cls)value);
		} else if (value instanceof Instance) {
			owlValue = copyInstance (owlModel, (Instance)value);
		}
		return owlValue;
	}

	public static Instance copyInstance (JenaOWLModel owlModel, Instance instance) {
		Cls type = instance.getDirectType();
		OWLNamedClass owltype = FMAtoOWL.getOwlClass(type);
//Log.getLogger().info("type = " + type + "owltypa = " + owltype);
		
		String newName = FMAtoOWL.getValidOWLName(instance.getName());
		RDFIndividual existing = owlModel.getRDFIndividual(newName);
		while (existing != null) {
			newName += "_";
			existing = owlModel.getRDFIndividual(newName);
		}
	
		//******** creates the individual in the wrong model
		//OWLIndividual result = owltype.createOWLIndividual(FMAtoOWL.getValidOWLName(newName));
		Instance result = owlModel.createInstance(newName, owltype);
		
		copyOwnSlotValues (owlModel, instance, (OWLIndividual)result);
		
		return result;
	}


}
